/***************************************************************************
 *   Copyright (C) 2005 by Ivan Cukic                                      *
 *   mr02014@alas.matf.bg.ac.yu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *  Changelog (dctmatrix.h and dctmatrix.cpp):
 *    2005.06.16.
 *      dctMatrix klasa za kvadratne matrice.
 *    2005.28.07.
 *      Modifikovanje klase da podrzava i nekvadratne matrice.
 */


#include "dctmatrix.h"

void dctMatrix::setSize(int Size) {
  /*destroyData();
  rowCount = Size;
  colCount = Size;
  createData();*/
  setSize(Size, Size);
}

void dctMatrix::setSize(int Rows, int Cols) {
  destroyData();
  rowCount = Rows;
  colCount = Cols;
  createData();
}

float& dctMatrix::get(int row, int col) {
  outOfBounds = 0;
  if (
    (row >= rowCount) ||
    (row < 0) ||
    (col >= colCount) ||
    (col < 0)
  ) {
    outOfBounds = 1;
    return outOfBoundsValue;
  }
  
  return *(matrixData + colCount * row + col);
}

void dctMatrix::copy(dctMatrix *m) {
  if (m == NULL) return;
  if (m == this) return;
  m->setSize(rowCount, colCount);
  for (int i = 0; i < rowCount; i++)
    for (int j = 0; j < colCount; j++)
      m->get(i, j) = get(i, j);
}

void dctMatrix::init() {
  outOfBoundsValue = 0;
  setAll(0);
}

void dctMatrix::setAll(float value) {
  for (int u = 0; u < rowCount; u++)
    for (int v = 0; v < colCount; v++)
      get(u, v) = value;
}

void dctMatrix::dct(int i) {
  float pi = 3.141592653579;
  
  float alpha_p_0 = 1.0 / sqrt(rowCount);
  float alpha_p_n = sqrt(2.0 / rowCount);
  
  float alpha_q_0 = 1.0 / sqrt(colCount);
  float alpha_q_n = sqrt(2.0 / colCount);
  
  dctMatrix tmp(rowCount, colCount);
  copy(&tmp);
  
  init();
  
  if (i >= 0) {
    /* DCT */
    for (int u = 0; u < rowCount; u++) {
      for (int v = 0; v < colCount; v++) {
        float alpha_p = (u == 0)?alpha_p_0:alpha_p_n;
        float alpha_q = (v == 0)?alpha_q_0:alpha_q_n;

        for (int x = 0; x < rowCount; x++) {
          for (int y = 0; y < colCount; y++) {
            get(u, v) += tmp.get(x, y) *
              cos (pi * u * (2 * x + 1) / (2 * rowCount)) *
              cos (pi * v * (2 * y + 1) / (2 * colCount));
          }
        }
      
        get(u, v) *= alpha_p * alpha_q;
      }
    }
  } else {
    /* Inverzna DCT */
    for (int x = 0; x < rowCount; x++) {
      for (int y = 0; y < colCount; y++) {
        for (int u = 0; u < rowCount; u++) {
          for (int v = 0; v < colCount; v++) {
            float alpha_p = (u == 0)?alpha_p_0:alpha_p_n;
            float alpha_q = (v == 0)?alpha_q_0:alpha_q_n;
            get(x, y) += alpha_p * alpha_q * tmp.get(u, v) *
              cos (pi * u * (2 * x + 1) / (2 * rowCount)) *
              cos (pi * v * (2 * y + 1) / (2 * colCount));
          }
        }
      }
    }
  }
}

void dctMatrix::createData() {
  matrixData = NULL;
  outOfBounds = 0;
  
  if ((rowCount > 0) && (colCount > 0))
    matrixData = (float *)malloc(rowCount * colCount * sizeof(float));
  if (matrixData == NULL) { rowCount = 0; colCount = 0; };
}

void dctMatrix::destroyData() {
  if (matrixData != NULL)
    free(matrixData);
}


