/***************************************************************************
 *   Copyright (C) 2005 by Ivan Cukic                                      *
 *   mr02014@alas.matf.bg.ac.yu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DCTMATRIX_H
#define DCTMATRIX_H

/**
@author Ivan Cukic
*/
#include <cstdlib>
#include <cmath>
#include <iostream>
class dctMatrix {
  public:
    /** dctMatrix() - podrazumevani konstruktor
     *  Kreira matricu dimenzija 0x0
     */
    dctMatrix() : outOfBounds(0), matrixData(0) { setSize(0); }
    
    /** dctMatrix(int Size)
     *  Kreira kvadratnu matricu matricu dimenzija @arg(Size) x @arg(Size)
     */
    dctMatrix (int Size) : outOfBounds(0), matrixData(0) { setSize(Size); }
    
    /** dctMatrix(int Rows, int Cols)
     *  Kreira matricu sa @arg(Rows) vrsta i @arg(Cols) kolona
     */
    dctMatrix (int Rows, int Cols) : outOfBounds(0), matrixData(0) { setSize(Rows, Cols); }
    
    /** ~dctMatrix() - destruktor */
    ~dctMatrix() { destroyData(); }
    
    /** void setSize(int Size)
     *  Unistava stare podatke o matrici (oslobadja memoriju)
     *  Alocira novu memoriju za matricu velicine @arg(Size) * @arg(Size)
     */
    void setSize(int Size);
    
    /** void setSize(int Rows, int Cols)
     *  Unistava stare podatke o matrici (oslobadja memoriju)
     *  Alocira novu memoriju za matricu velicine @arg(Rows) * @arg(Cols)
     */
    void setSize(int Rows, int Cols);

    /** int getRowCount()
     *  Vraca broj vrsta matrice
     */
    int getRowCount() { return rowCount; }
    
    /** int getColCount()
     *  Vraca broj kolona matrice
     */
    int getColCount() { return colCount; }
    
    /** float& get(int row, int col)
     *  Vraca referencu na element koji se nalazi u preseku
     *  @arg(row) vrste i @arg(col) kolone.
     *
     *  U slucaju da ne postoji polje koje je trazeno, vraca se nula,
     *  a outOfBounds se postavlja na 1
     */
    float& get(int row, int col);
    
    /** void copy(dctMatrix *m)
     *  Kopira svoj sadrzaj u matricu @arg(m)
     */
    void copy(dctMatrix *m);

    /** void setAll(float value)
     *  Postavlja vrednosti svih polja matrice na @arg(value)
     */
    void setAll(float value);
    
    /** void dct(int i)
     *  Zamenjuje svoje vrednosti vrednostima izracunate DCT
     *  (u slucaju kad je @arg(i) >= 0) ili inverzne DCT
     *  (u slucaju kad je @arg(i) < 0) nad samom sobom
     */
    void dct(int i);
    
    /** void init()
     *  Postavlja sva polja u matrici na nulu
     */
    void init();

    /** int outOfBounds
     *  Videti dokumentaciju za get(int row, int col)
     */
    int outOfBounds;
  protected:
    float outOfBoundsValue;
    int rowCount, colCount;
    float *matrixData;
    void createData();
    void destroyData();
  private:
    dctMatrix(dctMatrix& dm);
    void operator = (dctMatrix& dm);
};


#endif
