/***************************************************************************
 *   Copyright (C) 2005 by Ivan Cukic                                      *
 *   mr02014@alas.matf.bg.ac.yu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kspliram.h"
#include "frmMain.h"

#include <qlabel.h>

#include <kmainwindow.h>
#include <klocale.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qstring.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include "dctmatrix.h"
#include <kaboutdialog.h>

kspliram::kspliram(QWidget *parent=0, const char *name=0) : frmMain(parent, name) {
  dlgAbout = new KAboutDialog();
  dlgAbout->setTitle("About kSpliram");
  dlgAbout->setProduct("kSpliram", "0.7", "Ivan Cukic", "2005");
  dlgAbout->setAuthor ("Ivan Cukic", "mr02014@alas.matf.bg.ac.yu", "http://alas.matf.bg.ac.yu/~mr02014/", "This program is distributed under the terms of GNU General Public License version 2 or later.");
  dlgAbout->setVersion ("0.7 (Astangika) Release Candidate 3");
  dlgAbout->addLicensePage ("Licensing info", "This program is distributed under the terms of GNU General Public License version 2 or later.");
  matrix.setSize(3, 3);
  matrix.setAll(0.0);
  reloadTheTable();
}

kspliram::~kspliram() {
  delete dlgAbout;
}

void kspliram::btnOpen_clicked() {
  QString s = QFileDialog::getOpenFileName(
              "",
              "KSD File (*.ksd)",
              this,
              "Read Matrix from:",
              "Choose a file to open" );
  if (s != QString::null) {
    std::cerr << "kSpliram [status]: loading the matrix from [" << s << "]" << std::endl;
    std::ifstream in(s);
    int rc, cc;
    in >> rc;
    in >> cc;
    matrix.setSize(rc, cc);

    float read;

    for (int i = 0; i < rc; i++) {
      for (int j = 0; j < cc; j++) {
        in >> read;
        matrix.get(i, j) = read;
      }
    }

    reloadTheTable();
  } else
    std::cerr << "kSpliram [status]: loading canceled" << std::endl;

}

void kspliram::btnSave_clicked() {
  QString s = QFileDialog::getSaveFileName(
                    "",
                    "KSD File (*.ksd)",
                    this,
                    "Save Matrix to:"
                    "Choose a filename to save under" );
  if (s != QString::null) {
    std::cerr << "kSpliram [status]: saving the matrix to [" << s << "]" << std::endl;
    std::ofstream out(s);
    reloadTheMatrix();

    out << matrix.getRowCount() << " " << matrix.getColCount() << std::endl;
    for (int i = 0; i < matrix.getRowCount(); i++) {
      for (int j = 0; j < matrix.getColCount(); j++) {
        if (j != 0) out << " ";
        out << matrix.get(i, j);
      }
      out << std::endl;
    }
  } else
    std::cerr << "kSpliram [status]: saving canceled" << std::endl;

}

void kspliram::btnNew_clicked() {
  int rows = stringToInt(txtNewRowCount->text());
  int cols = stringToInt(txtNewColCount->text());
  matrix.setSize(rows, cols);
  matrix.setAll(0.0);
  reloadTheTable();
  txtNewRowCount->setText(intToString(rows));
  txtNewColCount->setText(intToString(cols));

}

void kspliram::btnSetAll_clicked() {
  float value = stringToFloat(txtSetAll->text());
  matrix.setAll(value);
  reloadTheTable();
  txtSetAll->setText(floatToString(value));
}

void kspliram::btnValidate_clicked() {
  reloadTheMatrix();
  reloadTheTable();
}

void kspliram::btnDCT_clicked() {
  reloadTheMatrix();
  matrix.dct(1);
  reloadTheTable();
}

void kspliram::btnDCTInv_clicked() {
  reloadTheMatrix();
  matrix.dct(-1);
  reloadTheTable();
}

void kspliram::btnAbout_clicked() {
  dlgAbout->show();
}

void kspliram::btnExit_clicked() {
  exit(EXIT_SUCCESS);
}

void kspliram::reloadTheMatrix() {
  int rows, cols;
  rows = tblMatrix->numRows();
  cols = tblMatrix->numCols();

  matrix.setSize(rows, cols);
  for (int row = 0; row < rows; row++) {
    for (int col = 0; col < rows; col++) {
      matrix.get(row, col) = stringToFloat(tblMatrix->text(row, col));
    }
  }
}

void kspliram::reloadTheTable() {
  int rows, cols;
  rows = matrix.getRowCount();
  cols = matrix.getColCount();

  tblMatrix->setNumRows(rows);
  tblMatrix->setNumCols(cols);

  for (int row = 0; row < rows; row++) {
    for (int col = 0; col < rows; col++) {
      tblMatrix->setText(row, col, floatToString(matrix.get(row, col)));
    }
  }
}

float kspliram::stringToFloat(std::string val) {
  std::stringstream ssresult (std::stringstream::in | std::stringstream::out);
  float result = 0;
  ssresult << val;
  ssresult >> result;
  return result;
}

int kspliram::stringToInt(std::string val) {
  std::stringstream ssresult (std::stringstream::in | std::stringstream::out);
  int result = 0;
  ssresult << val;
  ssresult >> result;
  return result;
}

std::string kspliram::floatToString(float val) {
  std::stringstream ssresult (std::stringstream::in | std::stringstream::out);
  ssresult << val;
  return ssresult.str();
}

std::string kspliram::intToString(int val) {
  std::stringstream ssresult (std::stringstream::in | std::stringstream::out);
  ssresult << val;
  return ssresult.str();
}


#include "kspliram.moc"
